<?php
/**
 * MNMLD_About_Me widget class
 *
 * @since 1.0
 */
 
class MNMLD_About_Me extends WP_Widget {

	var $defaults;		// default values


    // Create the widget options and such here, then call
    // $this->WP_Widget to let wordpress know about it.
    function __construct() {
		
		$this->defaults[ 'widget_slug' ]		= 'mnmld-about-me'; 	// widget slug
		
		$this->defaults[ 'widget_title' ]		= 'About me'; 			// widget title
		 
		//Language support
		$language_codes = explode( '_', get_locale() );
		switch ( $language_codes[ 0 ] ) {
			default:
				$aboutme_widget_name = 'MNMLD - About Me';
				$widget_desc = 'About me widget';
		}

		// Domain name and protocol of WP site
		$parsed_url = parse_url( home_url() );
		$this->defaults[ 'site_protocol' ]		= $parsed_url[ 'host' ];
		$this->defaults[ 'site_url' ]			= $parsed_url[ 'scheme' ];
		unset( $parsed_url );
		
		// other vars
		$widget_ops  = array( 'classname' => $this->defaults[ 'widget_slug' ], 'description' => $widget_desc );
		parent::__construct( $this->defaults[ 'widget_slug' ], $aboutme_widget_name, $widget_ops );

		add_action( 'save_post',				array( $this, 'flush_widget_cache' ) );
		add_action( 'deleted_post',				array( $this, 'flush_widget_cache' ) );
		add_action( 'switch_theme',				array( $this, 'flush_widget_cache' ) );

	}

    // display the widget on the front end.
	function widget( $args, $instance ) {

		if ( ! isset( $args[ 'widget_id' ] ) ) {
			$args[ 'widget_id' ] = $this->id;
		}

		// get and sanitize values
		$title			  = ( ! empty( $instance[ 'title' ] ) ) ? $instance[ 'title' ] : $this->defaults[ 'widget_title' ];
		$title			  = apply_filters( 'widget_title', $title, $instance, $this->id_base );
		
		$subtitle			  = ( ! empty( $instance[ 'subtitle' ] ) ) ? $instance[ 'subtitle' ] : '';
		$text_desc			  = ( ! empty( $instance[ 'text_desc' ] ) ) ? $instance[ 'text_desc' ] : '';
		
		$img_url			  = ( ! empty( $instance[ 'img_url' ] ) ) ? $instance[ 'img_url' ] : '';
		
		$show_button		  = ( ! empty( $instance[ 'show_button' ] ) ) ? $instance[ 'show_button' ] : '';
		$button_text		  = ( ! empty( $instance[ 'button_text' ] ) ) ? $instance[ 'button_text' ] : '';
		$button_url			  = ( ! empty( $instance[ 'button_url' ] ) ) ? $instance[ 'button_url' ] : '';
		
		// print list
		include 'widget.php';

	}

    // The widget form in the admin area.
	function update( $new_widget_settings, $old_widget_settings ) {
		
		$instance = $old_widget_settings;
		
		// sanitize user input before update
		$instance[ 'title' ] 			= ( isset( $new_widget_settings[ 'title' ] ) )          ? strip_tags( $new_widget_settings[ 'title' ] )				: $this->defaults[ 'widget_title' ];
		$instance[ 'subtitle' ] 		= ( isset( $new_widget_settings[ 'subtitle' ] ) )     	? strip_tags( $new_widget_settings[ 'subtitle' ] )			: '';
		$instance[ 'text_desc' ] 		= ( isset( $new_widget_settings[ 'text_desc' ] ) )      ? strip_tags( $new_widget_settings[ 'text_desc' ] )			: '';
		
		$instance[ 'img_url' ] 			= ( isset( $new_widget_settings[ 'img_url' ] ) )        ? strip_tags( $new_widget_settings[ 'img_url' ] )			: '';
		
		$instance[ 'show_button' ] 		= ( isset( $new_widget_settings[ 'show_button' ] ) )     ? strip_tags( $new_widget_settings[ 'show_button' ] )		: '';
		$instance[ 'button_text' ] 		= ( isset( $new_widget_settings[ 'button_text' ] ) )     ? strip_tags( $new_widget_settings[ 'button_text' ] )		: '';
		$instance[ 'button_url' ] 		= ( isset( $new_widget_settings[ 'button_url' ] ) )     ? strip_tags( $new_widget_settings[ 'button_url' ] )		: '';
		
		
		// empty widget cache
		$this->flush_widget_cache();

		$alloptions = wp_cache_get( 'alloptions', 'options' );

        // return sanitized current widget settings
		return $instance;
	}

	function flush_widget_cache() {
		wp_cache_delete( $this->defaults[ 'widget_slug' ], 'widget' );
	}

    // Saves the  data, called via AJAX. But the saving logic comes here.
	function form( $instance ) {
		// get and sanitize values
		$title	= ( isset( $instance[ 'title' ] ) ) ? $instance[ 'title' ]	: $this->defaults[ 'widget_title' ];
		$subtitle	= ( isset( $instance[ 'subtitle' ] ) ) ? $instance[ 'subtitle' ]	: '';
		$text_desc	= ( isset( $instance[ 'text_desc' ] ) ) ? $instance[ 'text_desc' ]	: '';
		
		$img_url	= ( isset( $instance[ 'img_url' ] ) ) ? $instance[ 'img_url' ]	: '';
		
		$show_button	= ( isset( $instance[ 'show_button' ] ) ) ? $instance[ 'show_button' ]	: '';
		$button_text	= ( isset( $instance[ 'button_text' ] ) ) ? $instance[ 'button_text' ]	: '';
		$button_url	= ( isset( $instance[ 'button_url' ] ) ) ? $instance[ 'button_url' ]	: '';
		
		// compute ids only once to improve performance
		$field_ids = array();
		$field_ids[ 'title' ]				= $this->get_field_id( 'title' );
		$field_ids[ 'subtitle' ]			= $this->get_field_id( 'subtitle' );
		$field_ids[ 'text_desc' ]			= $this->get_field_id( 'text_desc' );
		
		$field_ids[ 'img_url' ]				= $this->get_field_id( 'image_url' );
		
		$field_ids[ 'show_button' ]			= $this->get_field_id( 'show_button' );
		$field_ids[ 'button_text' ]			= $this->get_field_id( 'button_text' );
		$field_ids[ 'button_url' ]			= $this->get_field_id( 'button_url' );
		
		// print form in widgets page
		include 'form.php';

	}
	
}


// Register widget on init
function register_MNMLD_About_Me () {
	register_widget( 'MNMLD_About_Me' );
}

add_action( 'widgets_init', 'register_MNMLD_About_Me', 1 );