<?php
/**
 *	Design Metaboxes
 *	
 * 	Registered metaboxes for post and page layouts.
 *
 * 	@version 1.2.0
 *	@since	 1.0.0
 *	@package mnmld-themes
 *	@author	 mnmld
 *
 **/

//Exit if file accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

// POST Metaboxes ---------------------------------------------------------------------------------

// Define Metabox
function mnmld_metabox_add_post() {

	add_meta_box(
		'mnmld_metabox_post_layout', // metabox ID
		 __( 'Post Layout Settings', 'mnmld-themes' ), // title
		'mnmld_metabox_post_layout_callback', // callback function
		'post', // post type or post types in array
		'side', // position (normal, side, advanced)
		'high' // priority (default, low, high, core)
	);

}

add_action( 'add_meta_boxes', 'mnmld_metabox_add_post' );

// Callback
function mnmld_metabox_post_layout_callback( $object ) {
	global $post;
	
    wp_nonce_field(basename(__FILE__), 'meta-box-nonce' );

    ?>
    
	    <div class="mnmld_metabox_div">
		
            <h4 style="margin-bottom:0;"><?php _e('Post Layout Design', 'mnmld-themes');?></h4>
			
			<p style="margin-top:0; padding-top:0; color:#777;"><?php _e('Select your post page layout design. Leave (none) to use assigned design layout in customizer.', 'mnmld-themes');?></p>
			
            <select name="mnmld_metabox_post_select">
                <?php 
				// Define MNMLD theme prefix
				$mnmld_theme_prefix = __MNMLD_THEME_PREFIX; 
				
				if( $mnmld_theme_prefix == 'devnode' ) {
					// Theme single layouts
					$option_values = array( '', 'blog', 'modern', 'cover', 'floating', 'feature', 'classic' );
				}else{
					// Theme single layouts
					$option_values = array( '', 'modern', 'blog', 'feature', 'cover', 'magazine', 'editorial', 'floating', '2cols', 'classic',  'aligned', 'wide' );
				}
			
                foreach( $option_values as $key => $layout_id ) {
					
					if( $layout_id == '' ) { 
						$layout_name = esc_attr__( '- none -', 'mnmld-themes' );
					}elseif( $layout_id == 'wide' ){ 
						$layout_name = esc_attr__( 'Full-Width', 'mnmld-themes' ); 						
					}elseif( $layout_id == '2cols' ){ 
						$layout_name = esc_attr__( '2 Columns', 'mnmld-themes' );  
					}else{
						$layout_name = ucfirst( $layout_id );
					}
						
					if( $layout_id == get_post_meta( $object->ID, "mnmld_metabox_post_select", true ) ) {
                    ?>	
                    <option value="<?php echo esc_attr( $layout_id ); ?>" selected><?php echo esc_html( $layout_name ); ?></option>
                    <?php    
                    }else{
					?>
                    <option value="<?php echo esc_attr( $layout_id ); ?>"><?php echo esc_html( $layout_name ); ?></option>
                    <?php
                    }
                }
                ?>
            </select>

        </div>
		
    <?php  
}

// Save
function mnmld_metabox_post_layout_save( $post_id, $post, $update ) {
	
	global $post;
	
    if ( ! isset( $_POST['meta-box-nonce'] ) || ! wp_verify_nonce( $_POST['meta-box-nonce'], basename(__FILE__) ) )
        return $post_id;

    if( ! current_user_can( 'edit_post', $post_id ) ) {
        return $post_id;
	}
	
    if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
        return $post_id;
	}
	
    $slug = 'post';
	
    if( $slug != $post->post_type )
        return $post_id;


    $meta_box_dropdown_value = '';
 
    if( isset( $_POST['mnmld_metabox_post_select'] ) ){
        $meta_box_dropdown_value = $_POST['mnmld_metabox_post_select'];
    }  
	
	// Update
    update_post_meta( $post_id, 'mnmld_metabox_post_select', $meta_box_dropdown_value );

}

add_action( 'save_post', 'mnmld_metabox_post_layout_save', 10, 3 );




// PAGE Metaboxes ---------------------------------------------------------------------------------

// Define Metabox
function mnmld_page_metabox_add_post() {
	
	// 
	add_meta_box(
		'mnmld_page_metabox_post_layout', // metabox ID
		 __( 'Page Design', 'mnmld-themes' ), // title
		'mnmld_page_metabox_post_layout_callback', // callback function
		'page', // post type or post types in array
		'side', // position (normal, side, advanced)
		'high' // priority (default, low, high, core)
	);
	
	add_meta_box(
		'mnmld_page_metabox_sidebar', // metabox ID
		 __( 'Page Sidebar', 'mnmld-themes' ), // title
		'mnmld_page_metabox_sidebar_callback', // callback function
		'page', // post type or post types in array
		'side', // position (normal, side, advanced)
		'high' // priority (default, low, high, core)
	);

}

add_action( 'add_meta_boxes', 'mnmld_page_metabox_add_post' );

// Callbacks -----------------------------------------------------------------------------------------
// Page Layouts
function mnmld_page_metabox_post_layout_callback( $object ) {
	global $post;
	
    wp_nonce_field(basename(__FILE__), 'meta-box-nonce' );

    ?>
    
	    <div class="mnmld_page_metabox_div">
		
            <h4 style="margin-bottom:0;"><?php _e('Page Layout Design', 'mnmld-themes');?></h4>
			
			<p style="margin-top:0; padding-top:0; color:#777;"><?php _e('Select your page layout design. Leave (none) to use assigned design layout in customizer.', 'mnmld-themes');?></p>
			
            <select name="mnmld_page_metabox_post_select">
                <?php 
				
				// Theme single layouts
				$option_values = array( '', 'default', 'aligned', 'wide', 'heading', 'cover', 'feature', 'floating' );

                foreach( $option_values as $key => $layout_id ) {
					
					if( $layout_id == '' ) { 
						$layout_name = esc_attr__( '- none -', 'mnmld-themes' );
					}elseif( $layout_id == 'wide' ){ 
						$layout_name = esc_attr__( 'Full-Width', 'mnmld-themes' ); 						
					}else{
						$layout_name = ucfirst( $layout_id );
					}
						
					if( $layout_id == get_post_meta( $object->ID, 'mnmld_page_metabox_post_select', true ) ) {
                    ?>	
                    <option value="<?php echo esc_attr( $layout_id ); ?>" selected><?php echo esc_html( $layout_name ); ?></option>
                    <?php    
                    }else{
					?>
                    <option value="<?php echo esc_attr( $layout_id ); ?>"><?php echo esc_html( $layout_name ); ?></option>
                    <?php
                    }
                }
                ?>
            </select>

        </div>
		
    <?php  
}


function mnmld_page_metabox_sidebar_callback( $object ) {
	global $post;
	
    wp_nonce_field(basename(__FILE__), 'meta-box-nonce' );

    ?>
    
	    <div class="mnmld_page_metabox_div">
		
            <h4 style="margin-bottom:0;"><?php _e('Page Sidebar', 'mnmld-themes');?></h4>
			
			<p style="margin-top:0; padding-top:0; color:#777;"><?php _e('Select your page sidebar alignment. Leave (none) to use assigned design layout in customizer.', 'mnmld-themes');?></p>
			
            <select name="mnmld_page_metabox_sidebar_select">
                <?php 
				
				// Theme sidebar layouts
				$option_values = array( '', 'no-sidebar', 'sidebar-right', 'sidebar-left' );

                foreach( $option_values as $key => $sidebar_id ) {
					
					if( $sidebar_id == '' ) { 
						$sidebar_name = esc_attr__( '- none -', 'mnmld-themes' );
					}elseif( $sidebar_id == 'no-sidebar' ){ 
						$sidebar_name = esc_attr__( 'No Sidebar', 'mnmld-themes' ); 
					}elseif( $sidebar_id == 'sidebar-right' ){ 
						$sidebar_name = esc_attr__( 'Right Sidebar', 'mnmld-themes' ); 
					}elseif( $sidebar_id == 'sidebar-left' ){ 
						$sidebar_name = esc_attr__( 'Left Sidebar', 'mnmld-themes' ); 
					}else{
						$sidebar_name = ucfirst( $sidebar_id );
					}
						
					if( $sidebar_id == get_post_meta( $object->ID, 'mnmld_page_metabox_sidebar_select', true ) ) {
                    ?>	
                    <option value="<?php echo esc_attr( $sidebar_id ); ?>" selected><?php echo esc_html( $sidebar_name ); ?></option>
                    <?php    
                    }else{
					?>
                    <option value="<?php echo esc_attr( $sidebar_id ); ?>"><?php echo esc_html( $sidebar_name ); ?></option>
                    <?php
                    }
                }
                ?>
            </select>

        </div>
		
    <?php  
}

// Save ----------------------------------------------------------------------------------------------
function mnmld_page_metabox_post_layout_save( $post_id, $post, $update ) {
	
	global $post;
	
    if ( ! isset( $_POST['meta-box-nonce'] ) || ! wp_verify_nonce( $_POST['meta-box-nonce'], basename(__FILE__) ) )
        return $post_id;

    if( ! current_user_can( 'edit_post', $post_id ) ) {
        return $post_id;
	}
	
    if( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
        return $post_id;
	}
	
    $slug = 'page';
	
    if( $slug != $post->post_type )
        return $post_id;

	
	// Page Layout 
    $meta_box_dropdown_value = '';
	
    if( isset( $_POST['mnmld_page_metabox_post_select'] ) ){
        $meta_box_dropdown_value = $_POST['mnmld_page_metabox_post_select'];
    }  
	
	// Update page layout
    update_post_meta( $post_id, 'mnmld_page_metabox_post_select', $meta_box_dropdown_value );
	
	
	// Page Sidebar
    $meta_box_dropdown_value_sidebar = '';
	
    if( isset( $_POST['mnmld_page_metabox_sidebar_select'] ) ){
        $meta_box_dropdown_value_sidebar = $_POST['mnmld_page_metabox_sidebar_select'];
    }  
	
	// Update page layout
    update_post_meta( $post_id, 'mnmld_page_metabox_sidebar_select', $meta_box_dropdown_value_sidebar );
}

add_action( 'save_post', 'mnmld_page_metabox_post_layout_save', 10, 3 );